@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.c,v 1.237 2006/05/29 14:28:00 relson Exp $ */
+/* $Id: bogoconfig.c,v 1.243 2006/08/09 23:18:21 relson Exp $ */
 
 /*****************************************************************************
 
@@ -335,8 +335,12 @@ static const char *help_text[] = {
     "  --log-header-format               header written to log\n",
     "  --log-update-format               logged on update\n",
     "  --min-dev                         ignore if score near\n",
+    "  --min-token-len                   min len for single tokens\n",
+    "  --max-token-len                   max len for single tokens\n",
+    "  --max-multi-token-len             max len for multi-word tokens\n",
+    "  --multi-token-count               number of tokens per multi-word token\n",
     "  --ns-esf                          effective size factor for ham\n",
-    "  --replace-nonascii-characters     substitute '?' if bit8 is 1\n",
+    "  --replace-nonascii-characters     substitute '?' if bit 8 is 1\n",
     "  --robs                            Robinson's s parameter\n",
     "  --robx                            Robinson's x parameter\n",
     "  --sp-esf                          effective size factor for spam\n",
@@ -383,8 +387,8 @@ static void print_version(void)
     (void)fprintf(stdout,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2006 Eric S. Raymond,\n"
-		  "    David Relson, Matthias Andree, Greg Louis\n\n"
+		  "Copyright (C) 2002-2003 Eric S. Raymond, Adrian Otto, Gyepi Sam.\n"
+		  "Copyright (C) 2002-2006 David Relson, Matthias Andree, Greg Louis\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
 		  "redistribute it under the General Public License.  "
@@ -394,7 +398,7 @@ static void print_version(void)
 		  progtype, version, ds_version_str(), PACKAGE);
 }
 
-#define	OPTIONS	":-:bBc:Cd:DehHI:k:lL:m:MnNo:O:pqQRsStTuUvVx:X:y:"
+#define	OPTIONS	":-:bBc:Cd:DehHI:k:lL:m:MnNo:O:pPqQRsStTuUvVx:X:y:"
 
 /** These functions process command line arguments.
  **
@@ -701,6 +705,10 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     case O_HEADER_FORMAT:		header_format = get_string(name, val);			break;
     case O_LOG_HEADER_FORMAT:		log_header_format = get_string(name, val);		break;
     case O_LOG_UPDATE_FORMAT:		log_update_format = get_string(name, val);		break;
+    case O_MAX_TOKEN_LEN:		max_token_len=atoi(val);				break;
+    case O_MIN_TOKEN_LEN:		min_token_len=atoi(val);				break;
+    case O_MAX_MULTI_TOKEN_LEN:		max_multi_token_len=atoi(val);				break;
+    case O_MULTI_TOKEN_COUNT:		multi_token_count=atoi(val);				break;
     case O_REPLACE_NONASCII_CHARACTERS:	replace_nonascii_characters = get_bool(name, val);	break;
     case O_SPAMICITY_FORMATS:		set_spamicity_formats(val);				break;
     case O_SPAMICITY_TAGS:		set_spamicity_tags(val);				break;
