@@ -1,4 +1,4 @@
-/* $Id: bogoutil.c,v 1.226 2005/06/20 00:48:56 relson Exp $ */
+/* $Id: bogoutil.c,v 1.239 2006/08/09 23:18:21 relson Exp $ */
 
 /*****************************************************************************
 
@@ -18,7 +18,6 @@ AUTHORS:
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <unistd.h>
  
 #include "getopt.h"
 
@@ -68,7 +67,8 @@ static void ds_open_failure(bfpath *bfp, void *dbe)
     fprintf(stderr, "Error accessing file or directory '%s'.\n", bfp->filepath);
     if (errno != 0)
 	fprintf(stderr, "error #%d - %s.\n", errno, strerror(errno));
-    ds_cleanup(dbe);
+    if (dbe != NULL)
+	ds_cleanup(dbe);
     exit(EX_ERROR);
 }
 
@@ -89,13 +89,13 @@ static int ds_dump_hook(word_t *key, dsv_t *data,
     if (replace_nonascii_characters)
 	do_replace_nonascii_characters(key->text, key->leng);
 
-    printf( "%.*s %lu %lu",
+    fprintf( fpo, "%.*s %lu %lu",
 	    CLAMP_INT_MAX(key->leng), key->text,
 	    (unsigned long)data->spamcount,
 	    (unsigned long)data->goodcount);
     if (data->date)
-	printf(" %lu", (unsigned long)data->date);
-    putchar('\n');
+	fprintf( fpo, " %lu", (unsigned long)data->date);
+    fprintf( fpo, "\n");
 
     fflush(stdout); /* solicit ferror flag if output is shorter than buffer */
     return ferror(stdout) ? 1 : 0;
@@ -202,7 +202,8 @@ static ex_t load_wordlist(bfpath *bfp)
 	p = spanword(buf);
 	len = strlen((const char *)buf);
 
-	if (len > MAXTOKENLEN)
+	if (max_token_len != 0 &&
+	    len > max_token_len)
 	    continue;		/* too long - discard */
 
 	spamcount = (uint) atoi((const char *)p);
@@ -354,7 +355,7 @@ static ex_t display_words(bfpath *bfp, int argc, char **argv, bool show_probabil
 	robx = ROBX;
     }
 
-    printf(head_format, "", "spam", "good", "  Fisher");
+    fprintf(fpo, head_format, "", "spam", "good", "  Fisher");
     while (argc >= 0)
     {
 	dsv_t val;
@@ -384,11 +385,11 @@ static ex_t display_words(bfpath *bfp, int argc, char **argv, bool show_probabil
 		good_count = val.goodcount;
 
 		if (!show_probability)
-		    printf(data_format, token->text, spam_count, good_count);
+		    fprintf(fpo, data_format, token->text, spam_count, good_count);
 		else
 		{
 		    rob_prob = calc_prob(good_count, spam_count, msgcnts.goodcount, msgcnts.spamcount);
-		    printf(data_format, token->text, spam_count, good_count, rob_prob);
+		    fprintf(fpo, data_format, token->text, spam_count, good_count, rob_prob);
 		}
 		break;
 	    case 1:
@@ -432,6 +433,10 @@ static ex_t get_robx(bfpath *bfp)
 	dsv_t val;
 	word_t *word_robx = word_news(ROBX_W);
 
+	/* since compute_robinson_x() closes the wordlists, 
+	   init_wordlist() must be called again */
+	init_wordlist("word", bfp->filepath, 0, WL_REGULAR);
+
 	open_wordlists(DS_WRITE);
 
 	val.goodcount = 0;
@@ -443,7 +448,6 @@ static ex_t get_robx(bfpath *bfp)
 		begin_wordlist(word_lists);
 	    }
 	} while (ret == DS_ABORT_RETRY);
-	    
 
 	close_wordlists(true);
 	free_wordlists();
@@ -459,7 +463,7 @@ static void print_version(void)
     (void)fprintf(stdout,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2005 Gyepi Sam, David Relson, Matthias Andree\n\n"
+		  "Copyright (C) 2002-2006 Gyepi Sam, David Relson, Matthias Andree\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
 		  "redistribute it under the General Public License.  "
@@ -537,6 +541,14 @@ static const char *help_text[] = {
     "  --unicode=yes/no            - convert wordlist to/from unicode\n",
 #endif
     "\n",
+
+    "token parsing options:\n",
+    "  --min-token-len             - min len for single tokens\n",
+    "  --max-token-len             - max len for single tokens\n",
+    "  --max-multi-token-len       - max len for multi-word tokens\n",
+    "  --multi-token-count         - number of tokens per multi-word token\n",
+    "\n",
+
     NULL
     };
 
@@ -585,7 +597,7 @@ static struct option longopts_bogoutil[] = {
     { NULL,				0, 0, 0 }
 };
 
-#define	OPTIONS	":a:c:Cd:DhH:I:k:l:m:np:r:R:s:u:vVw:x:X:y:"
+#define	OPTIONS	":a:c:Cd:DhH:I:k:l:m:nO:p:r:R:s:u:vVw:x:X:y:"
 
 static int process_arglist(int argc, char **argv)
 {
@@ -593,6 +605,7 @@ static int process_arglist(int argc, char **argv)
     int count = 0;
 
     fpin = stdin;
+    fpo  = stdout;
     dbgout = stderr;
 
 #ifdef __EMX__
@@ -600,6 +613,9 @@ static int process_arglist(int argc, char **argv)
     _wildcard (&argc, &argv);	/* expand wildcards (*.*) */
 #endif
 
+    /* default: no token length checking */
+    max_token_len = 0;
+
     while (1)
     {
 	int option_index = 0;
@@ -616,6 +632,14 @@ static int process_arglist(int argc, char **argv)
 	count += process_arg(option, name, optarg);
     }
 
+    if (max_token_len != 0 && max_multi_token_len == 0) {
+	/* token length checking ... */
+	if (multi_token_count == 1)
+	    max_multi_token_len = max_token_len + MAX_PREFIX_LEN;
+	else
+	    max_multi_token_len = max_token_len = (max_token_len+1) * multi_token_count + MAX_PREFIX_LEN;
+    }
+
     if (count != 1)
     {
 	usage(stderr);
@@ -785,6 +809,14 @@ static int process_arg(int option, const char *name, const char *val)
 	}
 	break;
 
+    case 'O':
+	fpo = fopen(val, "wt");
+	if (fpo == NULL) {
+	    fprintf(stderr, "Can't write file '%s'\n", val);
+	    exit(EX_ERROR);
+	}
+	break;
+
     case 'D':
 	dbgout = stdout;
 	break;
@@ -799,6 +831,22 @@ static int process_arg(int option, const char *name, const char *val)
 	encoding = str_to_bool(val) ? E_UNICODE : E_RAW;
 	break;
 
+    case O_MAX_TOKEN_LEN:
+	max_token_len = atoi(optarg);
+	break;
+
+    case O_MIN_TOKEN_LEN:
+	min_token_len = atoi(optarg);
+	break;
+
+    case O_MAX_MULTI_TOKEN_LEN:
+	max_multi_token_len=atoi(optarg);
+	break;
+
+    case O_MULTI_TOKEN_COUNT:
+	multi_token_count=atoi(optarg);
+	break;
+
     default:
 	if (!dsm_options_bogoutil(option, &flag, &count, &ds_file, name, val)) {
 	    fprintf(stderr, "Invalid option '%s'\n", name);
@@ -813,7 +861,7 @@ static bfpath_mode get_mode(cmd_t cmd)
 {
     bfpath_mode mode = BFP_ERROR;
 
-    switch(cmd) {
+    switch (cmd) {
     case M_LOAD:
 	mode = BFP_MAY_CREATE;
 	break;
@@ -851,6 +899,7 @@ int main(int argc, char *argv[])
     fBogoutil = true;
 
     signal_setup();			/* setup to catch signals */
+    atexit(bf_exit);
 
     progtype = build_progtype(progname, DB_TYPE);
 
@@ -873,12 +922,19 @@ int main(int argc, char *argv[])
     bfpath_set_bogohome(bfp);
 
     mode = get_mode(flag);
+    if (bfpath_check_mode(bfp, mode)) {
+	if (bfp->isdir)
+	    bfpath_set_filename(bfp, WORDLIST);
+    }
+
     if (!bfpath_check_mode(bfp, mode)) {
 	fprintf(stderr, "Can't open wordlist '%s'\n", bfp->filepath);
 	exit(EX_ERROR);
     }
 
-    switch(flag) {
+    errno = 0;		/* clear error status */
+
+    switch (flag) {
 	case M_RECOVER:
 	    ds_init(bfp);
 	    rc = ds_recover(bfp, false);
